==> morphdb_utils-0.2.3-py3-none-any.whl: morphdb_utils/annotations.py <==
import base64
import io
import os
from typing import Literal, Optional

import matplotlib.figure  # type: ignore
import pandas as pd  # type: ignore
import plotly  # type: ignore
import plotly.io as pio  # type: ignore

# enum type: export format
VISUALIZATION_FORMAT = Literal["png", "html"]


def _get_html_from_mpl_image(
    fig: matplotlib.figure.Figure, format: VISUALIZATION_FORMAT = "html"
) -> str:
    buf = io.BytesIO()
    fig.savefig(buf, format="png")
    buf.seek(0)
    if format == "png":
        # return in base64 encoded png format
        return base64.b64encode(buf.read()).decode()
    elif format == "html":
        return f'<img src="data:image/png;base64,{base64.b64encode(buf.read()).decode()}" />'


def _get_html_from_plotly_image(fig, format: VISUALIZATION_FORMAT = "html") -> str:
    if format == "png":
        pio.kaleido.scope.chromium_args += ("--single-process",)
        buf = io.BytesIO()
        fig.write_image(buf, format="png")
        buf.seek(0)
        return base64.b64encode(buf.read()).decode()
    elif format == "html":
        buf = io.StringIO()
        fig.write_html(
            buf,
            include_plotlyjs="cdn",
            full_html=False,
            config={
                "modeBarButtonsToRemove": [
                    "zoom",
                    "pan",
                    "select",
                    "zoomIn",
                    "zoomOut",
                    "autoScale",
                    "resetScale",
                    "lasso2d",
                ],
                "displaylogo": False,
            },
        )
        buf.seek(0)
        return buf.getvalue()


def transform(output: Optional[str] = None):
    """decorator for main functions of morph's Python Cell.
    This decorator basically does nothing but indicates that the function perform some tabluar transformation
    and returns pandas DataFrame.
    """

    def decorator(func):
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            print(result)
            assert isinstance(result, pd.DataFrame)
            return result

        return wrapper

    return decorator


def visualize(library: Literal["matplotlib", "plotly"]):
    """decorator for main functions of morph's Python Cell.
    This decorator indicates that the function perform some visualization and returns Figure object.
    the Figure object will be converted to HTML string and returned.

    Args:
        library (Literal["matplotlib", "plotly"]): library used for visualization. either "matplotlib" or "plotly"
    """

    def decorator(func):
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)

            # post process
            format = os.getenv("MORPH_VISUALIZATION_FORMAT", "html")
            if library == "matplotlib":
                assert isinstance(result, matplotlib.figure.Figure)
                return _get_html_from_mpl_image(result, format)
            elif library == "plotly":
                assert isinstance(result, plotly.graph_objs._figure.Figure)
                return _get_html_from_plotly_image(result, format)
            else:
                raise ValueError("library should be either 'matplotlib' or 'plotly'")

        return wrapper

    return decorator


def report(func):
    """decorator for main functions of morph's Python Cell.
    This decorator indicates that the function perform some reporting and returns markdown string.
    """

    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        assert isinstance(result, str)
        return result

    return wrapper


def api(func):
    """decorator for main functions of morph's Python Cell.
    This decorator indicates that the function perform some API call and returns JSON string.
    """

    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        assert isinstance(result, dict)
        return result

    return wrapper

==> morphdb_utils-0.2.4-py3-none-any.whl: morphdb_utils/annotations.py <==
import base64
import io
import os
from typing import Callable, Literal, Optional

import matplotlib.figure  # type: ignore
import pandas as pd  # type: ignore
import plotly  # type: ignore
import plotly.io as pio  # type: ignore

# enum type: export format
VISUALIZATION_FORMAT = Literal["png", "html"]


def _get_html_from_mpl_image(
    fig: matplotlib.figure.Figure, format: VISUALIZATION_FORMAT = "html"
) -> str:
    buf = io.BytesIO()
    fig.savefig(buf, format="png")
    buf.seek(0)
    if format == "png":
        # return in base64 encoded png format
        return base64.b64encode(buf.read()).decode()
    elif format == "html":
        return f'<img src="data:image/png;base64,{base64.b64encode(buf.read()).decode()}" />'


def _get_html_from_plotly_image(fig, format: VISUALIZATION_FORMAT = "html") -> str:
    if format == "png":
        pio.kaleido.scope.chromium_args += ("--single-process",)
        buf = io.BytesIO()
        fig.write_image(buf, format="png")
        buf.seek(0)
        return base64.b64encode(buf.read()).decode()
    elif format == "html":
        buf = io.StringIO()
        fig.write_html(
            buf,
            include_plotlyjs="cdn",
            full_html=False,
            config={
                "modeBarButtonsToRemove": [
                    "zoom",
                    "pan",
                    "select",
                    "zoomIn",
                    "zoomOut",
                    "autoScale",
                    "resetScale",
                    "lasso2d",
                ],
                "displaylogo": False,
            },
        )
        buf.seek(0)
        return buf.getvalue()


def transform(output: Optional[str] = None):
    """Decorator for main functions of Morph's Python Cell.
    This decorator indicates that the function performs some tabular transformation
    and returns a pandas DataFrame.
    """

    def decorator(func: Callable):
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            assert isinstance(result, pd.DataFrame)
            return result, output

        return wrapper

    return decorator


def visualize(library: Literal["matplotlib", "plotly"]):
    """decorator for main functions of morph's Python Cell.
    This decorator indicates that the function perform some visualization and returns Figure object.
    the Figure object will be converted to HTML string and returned.

    Args:
        library (Literal["matplotlib", "plotly"]): library used for visualization. either "matplotlib" or "plotly"
    """

    def decorator(func):
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)

            # post process
            format = os.getenv("MORPH_VISUALIZATION_FORMAT", "html")
            if library == "matplotlib":
                assert isinstance(result, matplotlib.figure.Figure)
                return _get_html_from_mpl_image(result, format)
            elif library == "plotly":
                assert isinstance(result, plotly.graph_objs._figure.Figure)
                return _get_html_from_plotly_image(result, format)
            else:
                raise ValueError("library should be either 'matplotlib' or 'plotly'")

        return wrapper

    return decorator


def report(func):
    """decorator for main functions of morph's Python Cell.
    This decorator indicates that the function perform some reporting and returns markdown string.
    """

    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        assert isinstance(result, str)
        return result

    return wrapper


def api(func):
    """decorator for main functions of morph's Python Cell.
    This decorator indicates that the function perform some API call and returns JSON string.
    """

    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        assert isinstance(result, dict)
        return result

    return wrapper
